#!/usr/bin/perl

#=======================================================================#
# check_flexlm_perf_data.pl - check FLEXlm license servers     		#
#                     A Perl plugin for Nagios                 		#
#=======================================================================#
#                                                              		#
#                     check_flexlm_perf_data.pl v0.1           		#
#               based on LMstatparse 1.0 (c) ARM Ltd 2005.     		#
#                 Written by David Scanlan for ARM             		#
#                                                              		#
#	Usage:								#
#	-f (all|feature1,feature2...)	: features to include		#
#	-c (port@host)			: specify license server	#
#	-p (path_to_lmtuil)		: specify lmutil path		#
#									#
#########################################################################

use strict;
use threads;
use threads::shared;
use Getopt::Long;
use constant LOWDETAIL => '1';
use constant HIGHDETAIL => '2';

our($licserver);			# user specified license server (port@host)
our($lmutilpath);			# user specified path for lmutil
our($feature);				# store feature currently being worked on
our(%toolandlicversions) = ();		# array to hold which tools have which licenses
our($all);				# set to true (1) if all feature were selected
our($selected);				# set to true (1) if current feature has been selected
our(@chosenfeature);			# araay to hold a list of the chosen features
our($argument1);			# '-a' or '-f' dependent on a single feature chosen
our($argument2);			# only set for a single feature chosen
our($argument3);			# '-c' specifies license server
our($exitflag) : shared;		# exit flag set by thread to tell main loop to end
our(@logfilename);			# array used to hold output from lmutil lmstat
our($daemon);				# Vendor daemon
our($daemonline);			# damon line from output

# take some arguments from the command line
GetOptions ('c=s' => \$licserver,
			'f=s' => \@chosenfeature,
			'p=s' => \$lmutilpath);
@chosenfeature = split(/,/,join(',',@chosenfeature));	# allow multiple features to be selected

# set some defaults if variables not specified
if (!$lmutilpath)
{
	$lmutilpath = "lmutil";		# use default path to lmutil
}
elsif ($lmutilpath =~ s/\.exe$//)	# try and catch some common cases
{
}
elsif ($lmutilpath =~ s/\\$/\\lmutil/)
{
}
elsif ($lmutilpath =~ s/\/$/\/lmutil/)
{
}
else
{
	$lmutilpath =~ s/(.*)/$1\/lmutil/;
}
if (!$licserver)
{
	$licserver = $ENV{LM_LICENSE_FILE};		# look here if not on command line
	if (!$licserver)
	{
		$licserver = $ENV{ARMLMD_LICENSE_FILE};		# second look here
	}
}
if (!$licserver)
{
	print("Could not find license server...\n");
	print("Please specify one using -c <port\@host>\n");
	exit(3);
}
else
{
	$argument3 = "-c $licserver";
}

if (scalar(@chosenfeature) == 0)
{
	push(@chosenfeature,"compiler");
}

# loop through chosen features
foreach(@chosenfeature)
{
	my$chosenfeature = $_;
	$chosenfeature = lc($chosenfeature);	# put all features in lower case in array
	if($chosenfeature eq "all")				# check if all is specified as one of the features
	{
		$all = 1;
	}
}

if ($all == 1)
{
	$argument1 = "-a";
}
else
{
	if (scalar @chosenfeature == 1)		# only one element in array (not 'all')
	{
		$argument1 = "-f";		# only one feature selected so "lmutil lmstat -f ..." can be used
		$argument2 = @chosenfeature[0];
	}
	else
	{
		$argument1 = "-a";
	}
}

	my($licvers) = 0;
	print ("OK - I'm okay, are you okay? |");
	@logfilename = `$lmutilpath lmstat $argument1 $argument2 $argument3`;
	my $index;
	foreach my$line (@logfilename)
	{
		if ($line =~ m/^Feature usage info:/)
		{
			$daemonline = $index - 1;
			my $pos1 = index($logfilename[$daemonline],":");
			$daemon = substr($logfilename[$daemonline],0,$pos1);
			$daemon =~ s/[^\w]//g;
		}
		if ($line =~ m/^Users of (\w+|_): +\(Total of (\d+) license\w* issued; +Total of (\d+) license\w* in use\)/)
		{
			# could only detect end of section by start of next feature details
		 	%toolandlicversions = ();
			# only overwrite feature now extra details have been written
			$feature = $1;
			my($issued) = $2;
			my($inuse) = $3;
			$selected = 0;

			# set flag if feature was chosen at command line
			foreach (@chosenfeature)
			{
				my($feat)=$_;
				if ($feat eq $feature)
				{
					$selected = 1;
				}
			}

			# only do if the feature was selected at command line
			if (($selected == 1) || ($all == 1))
			{
				&OneLineSummary($daemon,$feature,$issued,$inuse);
			}
		}
		elsif ($line =~ m/\"($feature)\" v(\d+.\d+)/)
	    {
		    $licvers = $2;
	    }
		elsif ($line =~ m/\w+ \S+ \S+ \(v(\d+.\d+)\) \(\S+\/\d+ \d+\), start/)
		{
			my($toolvers) = $1;
			($toolandlicversions{"$feature License Vers: $licvers Tool Vers: $toolvers"}) += 1;			# increment count for a single vers of tools
		}
		elsif ($line =~ m/\w+ \S+ \S+ \(v(\d+.\d+)\) \(\S+\/\d+ \d+\) queued for (\d+)/)
		{
			my($toolvers) = $1;
			my($needed) = $2;
			($toolandlicversions{"queued for $feature License Vers: $licvers Tool Vers: $toolvers"}) += $needed;			# increment count for a queued licenses
		}
   		$index++;
	}

	@logfilename=();

# subroutine displays total tool licenses and how many are currently taken
sub OneLineSummary
{
	my($daemon) = $_[0];
	my($feature) = $_[1];
	my($issued) = $_[2];
	my($inuse) = $_[3];
	my($filename) = $feature . ".log";
	print ("$daemon\:\:$feature=$inuse;0;$issued ");
}
